<?php

/**
 * Medianetpay.com
 *
 * @author    Medianetpay
 * @copyright 2022 Medianetpay
 * @license   http://opensource.org/licenses/afl-3.0.php  Academic Free License (AFL 3.0)
 */

use PrestaShop\PrestaShop\Core\Payment\PaymentOption;

/*This checks for the Prestashop existence */

if (!defined('_PS_VERSION_')) {
    exit;
}

require_once(dirname(__FILE__) . '/classes/PaygolCustomClass.php');




class Medianetpay extends PaymentModule
{
    private $html       = '';
    private $postErrors = array();
    public $serviceID;
    public $secretKey;
    public $gateway     = 'https://plataforma.medianetpay.ec/secure/gangway/index.do';

    public function __construct()
    {
        $this->name                     = 'medianetpay';
        $this->tab                      = 'payments_gateways';
        $this->version                  = '1.0.1';
        $this->author                   = 'Medianetpay';
        $this->need_instance            = 1; //
        $this->currencies               = true;
        $this->currencies_mode          = 'checkbox';
        $this->module_key               = 'db7ec49b846681c1311e0aae5936de0e';
        $this->ps_versions_compliancy   = array('min' => '1.7', 'max' => _PS_VERSION_); //
        $this->bootstrap                = true;
        $config                         = Configuration::getMultiple(array('PAYGOL_SERVICEID', 'PAYGOL_SECRETKEY'));

        // Properties serviceid and secretkey get their values from the database.
        if (isset($config['PAYGOL_SERVICEID'])) {
            $this->serviceID = $config['PAYGOL_SERVICEID'];
        }
        if (isset($config['PAYGOL_SECRETKEY'])) {
            $this->secretKey = $config['PAYGOL_SECRETKEY'];
        }

        parent::__construct();

        switch ($this->context->language->iso_code) {
            case 'es':
                $iso = 'es';
                break;
            default:
                $iso = 'en';
        }

        $this->displayName  = $this->l('Medianetpay');
        $this->description  = $this->l('Medianetpay es un proveedor de pasarela de pagos que ');
        $this->description  .= $this->l('ofrece diferentes formas de pago locales e internacionales. ');
        //$this->description  .= $this->l('Additional information can be found at:');

        //    $this->description .= '<br><b><a href="https://plataforma.medianetpay.ec/'.$iso.'" target="_blank">'.
        //     ' https://plataforma.medianetpay.ec/'.$iso.'</a><br>
        //     <a href="https://plataforma.medianetpay.ec/'.$iso.'/pricing" target="_blank"> https://plataforma.medianetpay.ec/'.$iso.
        //     '/pricing</a></b>';
        //  $this->confirmUninstall = $this->l('Are you sure you want to uninstall?');

        if (!Configuration::get('PAYGOL')) {
            $this->warning = $this->l('No name provided');
        }
    }

    /**********************************************************************INSTALL**********************************************************************/
    // Possible transaction statuses, also register hooks.
    public function install()
    {
        if (Shop::isFeatureActive()) {
            Shop::setContext(Shop::CONTEXT_ALL);
        }
        //EN, ES, FR, PT
        $value1 = $this->addState(
            'Order Pending',
            'Orden Pendiente',
            'Ordre en Attente',
            'Pedido Pendente',
            '#FF8C00'
        );
        $value2 = $this->addState(
            'Processing Payment',
            'Procesando Pago',
            'Traitement du Paiement',
            'Processando o Pagamento',
            '#DDEEFF'
        );
        $value3 = $this->addState(
            'Successful',
            'Completado',
            'Terminé',
            'Concluído',
            '#32D600',
            true
        );

        if (!Configuration::get('PAYGOL_ORDER_PENDING')) {
            Configuration::updateValue('PAYGOL_ORDER_PENDING', $value1);
        }
        if (!Configuration::get('PAYGOL_ORDER_PROCESSING')) {
            Configuration::updateValue('PAYGOL_ORDER_PROCESSING', $value2);
        }
        if (!Configuration::get('PAYGOL_ORDER_SUCCESS')) {
            Configuration::updateValue('PAYGOL_ORDER_SUCCESS', $value3);
        }

        if (
            !parent::install() || !Configuration::updateValue('Medianetpay', 'Pagar con Medianetpay') ||
            !$this->registerHook('paymentOptions') || !$this->registerHook('paymentReturn')
        ) {
            return false;
        }

        return true;
    }

    private function addState($en, $es, $fr, $pt, $color, $paid = false)
    {
        $orderState = new OrderState();
        $orderState->name = array();
        foreach (Language::getLanguages() as $language) {
            if (Tools::substr($language['language_code'], 0, 2) == 'es') {
                $orderState->name[$language['id_lang']] = 'Medianetpay - ' . $es;
            } elseif (Tools::substr($language['language_code'], 0, 2) == 'fr') {
                $orderState->name[$language['id_lang']] = 'Medianetpay - ' . $fr;
            } elseif (Tools::substr($language['language_code'], 0, 2) == 'pt') {
                $orderState->name[$language['id_lang']] = 'Medianetpay - ' . $pt;
            } else {
                $orderState->name[$language['id_lang']] = 'Medianetpay - ' . $en;
            }
        }
        $orderState->send_email     = false;
        $orderState->module_name    = 'medianetpay';
        $orderState->color          = $color;
        $orderState->hidden         = false;
        $orderState->delivery       = false;
        $orderState->logable        = false;
        $orderState->paid           = $paid;

        if ($orderState->add()) {
            $data1 = dirname(__FILE__) . '/views/img/os_paygol.gif';
            $data2 = dirname(__FILE__) . '/../../img/os/' . (int)$orderState->id . '.gif';
            copy($data1, $data2);
            return $orderState->id;
        }
    }


    public function uninstall()
    {
        $config = Configuration::getMultiple(
            array('PAYGOL_ORDER_PENDING', 'PAYGOL_ORDER_PROCESSING', 'PAYGOL_ORDER_SUCCESS')
        );

        $PaygolCS = new PaygolCustomClass();

        if (
            !Configuration::deleteByName('PAYGOL_SERVICEID')
            || !Configuration::deleteByName('PAYGOL_SECRETKEY')
            || !$PaygolCS->deleteOrderState($config['PAYGOL_ORDER_PENDING'])
            || !$PaygolCS->deleteOrderState($config['PAYGOL_ORDER_SUCCESS'])
            || !$PaygolCS->deleteOrderState($config['PAYGOL_ORDER_PROCESSING'])
            || !Configuration::deleteByName('PAYGOL_ORDER_PENDING')
            || !Configuration::deleteByName('PAYGOL_ORDER_PROCESSING')
            || !Configuration::deleteByName('PAYGOL_ORDER_SUCCESS')
            || !parent::uninstall()
        ) {
            error_log('Uninstall error: check the nine lines before line ' . __LINE__ . ' in file ' . __FILE__);
            return false;
        }
        return true;
    }
    /***************************************************************************************************************************************************/
    /*****************************************************ADMIN PAGE*******************************************************/
    private function postValidation()
    {
        if (Tools::isSubmit('btnSubmit')) {
            if (!Tools::getValue('PAYGOL_SERVICEID')) {
                $this->postErrors[] = $this->l('You must enter your "Usuario"');
            }
            if (!Tools::getValue('PAYGOL_SECRETKEY')) {
                $this->postErrors[] = $this->l('You must enter your "Llave MD5"');
            }
        }
    }

    private function postProcess()
    {
        if (Tools::isSubmit('btnSubmit')) {
            Configuration::updateValue('PAYGOL_SERVICEID', Tools::getValue('PAYGOL_SERVICEID'));
            Configuration::updateValue('PAYGOL_SECRETKEY', Tools::getValue('PAYGOL_SECRETKEY'));
        }
        $this->_html .= $this->displayConfirmation($this->l('Settings updated'));
    }

    public function getConfigFieldsValues()
    {
        $value1 = Tools::getValue('PAYGOL_SERVICEID', Configuration::get('PAYGOL_SERVICEID'));
        $value2 = Tools::getValue('PAYGOL_SECRETKEY', Configuration::get('PAYGOL_SECRETKEY'));
        $value3 = Tools::getValue('PAYGOL_IPN', _PS_BASE_URL_ . __PS_BASE_URI__ . "modules/medianetpay/paygol_ipn.php");

        return array('PAYGOL_SERVICEID' => $value1, 'PAYGOL_SECRETKEY' => $value2, 'PAYGOL_IPN' => $value3,);
    }
    private function displayPaygol()
    {
        return $this->fetch($this->local_path . 'views/templates/admin/header.tpl');
    }

    public function renderForm()
    {
        $fields_form = array(
            'form' => array(
                'legend' => array(
                    'title' => $this->l('Medianetpay Configuracion'),
                    'icon' => 'icon-credit-card'
                ),
                'input' => array(
                    array(
                        'type' => 'text',
                        'class' => 'sm',
                        'label' => $this->l('Usuario:'),
                        'name' => 'PAYGOL_SERVICEID',
                        'required' => true
                    ),
                    array(
                        'type' => 'text',
                        'label' => $this->l('Llave MD5:'),
                        'name' => 'PAYGOL_SECRETKEY',
                        'required' => true
                    ),
                    /* array(
                        'type' => 'text',
                        'label' => $this->l('Payments notification URL (IPN):'),
                        'name' => 'PAYGOL_IPN',
                        'disabled' => true,
                        'desc' => $this->l('PASTE THIS URL INTO THE "NOTIFICATIONS" SECTION OF YOUR PAYGOL DASHBOARD'),
                    ),*/
                ),
                'submit' => array(
                    'title' => $this->l('Save'),
                )
            ),
        );

        $helper = new HelperForm();
        $helper->show_toolbar = false;
        $helper->table = $this->table;
        $lang = new Language((int)Configuration::get('PS_LANG_DEFAULT'));
        $helper->default_form_language = $lang->id;
        $value1 = Configuration::get('PS_BO_ALLOW_EMPLOYEE_FORM_LANG');
        $value2 = Configuration::get('PS_BO_ALLOW_EMPLOYEE_FORM_LANG');
        $helper->allow_employee_form_lang = $value1 ? $value2 : 0;
        $this->fields_form = array();
        $helper->id = (int)Tools::getValue('id_carrier');
        $helper->identifier = $this->identifier;
        $helper->submit_action = 'btnSubmit';
        $value3 = '&configure=' . $this->name . '&tab_module=' . $this->tab . '&module_name=' . $this->name;
        $helper->currentIndex = $this->context->link->getAdminLink('AdminModules', false) . $value3;
        $helper->token = Tools::getAdminTokenLite('AdminModules');
        $helper->tpl_vars = array(
            'fields_value' => $this->getConfigFieldsValues(),
            'languages' => $this->context->controller->getLanguages(),
            'id_language' => $this->context->language->id
        );

        return $helper->generateForm(array($fields_form));
    }

    public function getContent()
    {
        $this->_html = '';

        if (Tools::isSubmit('btnSubmit')) {
            $this->postValidation();
            if (!count($this->postErrors)) {
                $this->postProcess();
            } else {
                foreach ($this->postErrors as $err) {
                    $this->_html .= $this->displayError($err);
                }
            }
        }

        $this->_html .= $this->displayPaygol();
        $this->_html .= $this->renderForm();

        return $this->_html;
    }
    /***************************************************************************************************************************************************/
    /*******************************************************************CATALOG*************************************************************************/
    public function hookPaymentOptions()
    {
        if (!$this->active) {
            return;
        }

        $address = new Address($this->context->cart->id_address_delivery);

        $arguments = array(
            'serviceid'         => Tools::safeOutput(Configuration::get('PAYGOL_SERVICEID')),
            'total'             => $this->context->cart->getOrderTotal(),
            'currency'          => $this->context->currency->iso_code,
            'cart_id'           => $this->context->cart->id,
            'customer_id'       => $this->context->cart->id_customer,
            'secure_key'        => Tools::safeOutput(Configuration::get('PAYGOL_SECRETKEY')),
            'platform'          => "prestashop",
            'address'           => $address->address1 . " " . $address->address2,
            'phone'             => $address->phone,
            'city'             => $address->city,
            'platform_version'  => _PS_VERSION_,
            'return_cart'        => $this->context->link->getBaseLink() . "modules/medianetpay/response.php",
            'return_url'        => $this->context->link->getBaseLink() . "modules/medianetpay/validation.php"
        );

        $this->context->smarty->assign($arguments);

        /*CLEAN TEMPLATE (payment.tpl) CACHE */
        $this->context->smarty->compile_check = true;
        $this->context->smarty->force_compile = true;
        $this->context->smarty->clearCompiledTemplate();

        $newOption = new PaymentOption();
        $newOption->setModuleName($this->name)
            ->setCallToActionText($this->trans($this->l('Pagar con Medianetpay'), array(), 'Modules.Medianetpay.Shop'))
            //->setAction($this->context->link->getModuleLink('medianetpay', 'validation', array(), true))
            ->setAdditionalInformation($this->fetch('module:medianetpay/views/templates/front/payment.tpl'));

        $payment_options = array(
            $newOption,
        );

        return $payment_options;
    }

    // Used by paygol_ipn.php
    public function validationPaygol()
    {
        $serviceID      =    trim(Tools::getValue('service_id'));
        $secretKey      =    trim(Tools::getValue('key'));
        $price          =    trim(Tools::getValue('price'));
        $frmPrice       =    trim(Tools::getValue('frmprice'));
        $frmCurrency    =    trim(Tools::getValue('frmcurrency'));
        $custom         =    trim(Tools::getValue('custom'));
        $arrayCustom    =    explode("-", $custom);
        $cartID         =    trim($arrayCustom[0]);
        $customerID     =    trim($arrayCustom[1]);
        $userSecureKey  =    trim($arrayCustom[2]);
        $PS_ServiceID   =    trim(Tools::safeOutput(Configuration::get('PAYGOL_SERVICEID')));
        $PS_SecretKey   =    trim(Tools::safeOutput(Configuration::get('PAYGOL_SECRETKEY')));

        if ($PS_ServiceID != $serviceID || empty($serviceID)) {
            exit("Error: Wrong service ID");
        }
        if ($PS_SecretKey != $secretKey || empty($secretKey)) {
            exit("Error: Wrong key");
        }
        if (empty($price) || empty($frmPrice) || empty($frmCurrency)) {
            exit("Error: Wrong order values");
        }
        if (
            count($arrayCustom) < 3 || Tools::strlen($cartID) < 1 ||
            Tools::strlen($customerID) < 1 || Tools::strlen($userSecureKey) < 1
        ) {
            exit("Error: Wrong custom values");
        }
        if (Validate::isLoadedObject(new Cart($cartID))) {
            $ObjCart = new Cart($cartID);
        } else {
            exit("Error: Cart does not exist");
        }

        $priceValidation = $this->validateTotalPaid($cartID, $frmPrice, $frmCurrency);

        if ($ObjCart->orderExists() > 0 && $priceValidation == true) {
            $OrderObject = new Order();
            $OrderObject = new Order($OrderObject->getOrderByCartId($ObjCart->id));

            if ($userSecureKey != $OrderObject->secure_key || $userSecureKey != $ObjCart->secure_key) {
                exit("Error: Secure key does not match");
            }

            if ($customerID != $OrderObject->id_customer || $customerID != $ObjCart->id_customer) {
                exit("Error: Customer ID does not match");
            }

            if ((int)Configuration::get('PAYGOL_ORDER_SUCCESS') == $OrderObject->current_state) {
                exit("IPN CALLBACK: ALREADY COMPLETED");
            } else {
                $OrderObject->setCurrentState((int)Configuration::get('PAYGOL_ORDER_SUCCESS'));
                exit("IPN CALLBACK: OK");
            }
        }
    } //validationPaygol()

    /**********************************TOOLS**********************************/
    public function validateTotalPaid($cartID, $frmPrice, $frmCurrency)
    {

        $orderObject = new Order();
        $orderID = $orderObject->getOrderByCartId((int)$cartID);
        unset($orderObject);

        $orderObject = new Order($orderID);
        $totalPaid = $orderObject->total_paid;

        $currencyObject = new Currency($orderObject->id_currency);
        $currencyISOCode = $currencyObject->iso_code;

        $response = false;
        if ($frmCurrency == $currencyISOCode && $frmPrice == $totalPaid) {
            $response = true;
        } else {
            exit("CALLBACK ERROR: CHECK YOUR IPN AND THE AMOUNT FOR THIS TRANSACTION");
        }

        return ($response);
    }
}
